/* MCPU Projekt fuers Cyclone II Chinaboard
   Hauptmodul
   
   Umschalter EDIT/CPU sowie Display
*/

`default_nettype none       // das OPTION EXPLICIT in Verilog ;-)

`define EditMode 0
`define CPUMode  1

`include "utils/busmux.v"     // Bus-Multiplexer
`include "utils/debouncer.v"  // Entprellung fuer Taster und Umschalter
`include "moduls/DISPctrl.v"  // Steuerung 7-Seg-Anzeige
`include "moduls/Editor.v"    // Speicher-Editor
`include "moduls/MCPU.v"      // MCPU Minimal-Prozessor

module main (
 input  wire clk, // Systemtakt
 input  wire Switch1, // Umschalter EDIT/CPU
 input  wire Key1,  // Reset bzw. Init
 input  wire Key2,  // A-
 input  wire Key3,      // A+
 input  wire Key4,      // D+16
 input  wire Key5,      // D+
 output wire [7:0] seg,
 output wire [7:0] digit,
 output wire nCE,nOE,nWE,nBLE,nBHE,
 output wire [17:0] RamAddr,
 inout  wire [7:0] RamData
);

reg  [7:0] Data_in;
wire [5:0] Addr;
wire [7:0] Data;
wire [5:0] Addr1;   // vom Editor 
wire [7:0] Data1;
wire [5:0] Addr2;	  // von CPU
wire [7:0] Data2;
wire Modus; 
wire BtnInit;
wire Btn1, Btn2, Btn3, Btn4;
wire nOE1, nWE1, nOE2, nWE2;
wire Carry;
wire [7:0] code;
wire [5:0] PC;
wire [7:0] Accu;

 Debouncer SW1 (.clk(clk), .PB(Switch1), .PB_state(Modus) );
 Debouncer B1 (.clk(clk), .PB(Key1), .PB_down(BtnInit) );
 Debouncer B2 (.clk(clk), .PB(Key2), .PB_down(Btn1) );
 Debouncer B3 (.clk(clk), .PB(Key3), .PB_down(Btn2) );
 Debouncer B4 (.clk(clk), .PB(Key4), .PB_down(Btn3) );
 Debouncer B5 (.clk(clk), .PB(Key5), .PB_down(Btn4) );

 Editor Edit (.clk(clk), .mode(Modus), .init(BtnInit),
          .Adec(Btn1), .Ainc(Btn2), .DHinc(Btn3), .DLinc(Btn4), 
          .Addr(Addr1), .Data(Data1),
          .ramData(RamData),
          .nWE(nWE1), .nOE(nOE1) );
          
 DISPctrl Anzeige  (.clk(clk),
           .Addr(Addr),
           .Data(Data),
			  .PC(PC),
			  .Accu(Accu),
           .mode(Modus),
           .Carry(Carry),
           .Instr(code),
           .seg(seg),
           .digit(digit));

 MCPU CPU (.clk(clk), .mode(Modus), .reset(BtnInit), .sstep(Btn1),
           .ramData(RamData), .Addr(Addr2), .Carry(Carry), 
           .Instr(code), .Data(Data2), .nWE(nWE2), .nOE(nOE2), 
			  .Accu(Accu), .PC(PC) );

 BusMux #(16) Mux1 (.sel(Modus), 
           .InpA({Addr1,Data1,nWE1,nOE1}),
           .InpB({Addr2,Data2,nWE2,nOE2}),
           .Out({Addr,Data,nWE,nOE}) );
 
 // SRAM Steuerung
 assign nCE = 0;  // SRAM permanent aktiviert
 assign nBLE = 0; // vom 16-bit-SRAM nur niederwertiges Byte nutzen
 assign nBHE = 1;
 assign RamAddr = {12'b0,Addr};
 assign RamData = (~nWE & nOE) ? Data : 8'hZZ;

endmodule
