/* Display Controller fuer das MCPU System, steuert 8 stellige gemultiplexte
   7-Segment-Anzeige an.
   In den Modi Edit bzw. CPU wird die Anzeige unterschiedlich bedient:

   Edit:   "E"     ad.da   
   CPU:    "c"c.in.pc.ac

   mit c=Carry Flag, in=Instruction Code, ad=Adresse, da=Daten
	     pc=Program Counter, ac=Akkumulator
*/

`define Dot 8'h7F		// Maske fuer Zuschaltung Punkt

module DISPctrl (
 input  wire clk,
 input  wire [5:0] Addr,
 input  wire [7:0] Data,
 input  wire [5:0] PC,
 input  wire [7:0] Accu,
 input  wire mode,	   	// 0 fuer Edit, 1 fuer CPU
 input  wire Carry,			// carry flag der CPU
 input  wire [7:0] Instr,	// aktuelle Instruktion
 output reg  [7:0] seg,	   // Ziffern-Segmente
 output reg  [7:0] digit	// Ziffern-Position im 8-stelligen Display
);
 reg  [15:0] countdown;		// Zaehler für Display-Multiplexerei
 reg  [2:0]  Pos;				// Ziffernposition, Wertebereich {0 bis 7}

 always @ (posedge clk) begin

 countdown <= countdown - 1;
 if (countdown == 0) begin
   digit <= ~(1 << Pos);	// naechste Ziffer einschalten
	
	case(Pos)
	 0: if (mode) seg <= 8'd167;	// c fuer CPU 
	    else seg <= 8'd134;			// E fuer Edit	
	 1: if (mode) seg <= bin2hex({7'h0,Carry}) & `Dot;
            else seg <= 8'hFF;
	 2: if (mode) seg <= bin2hex(Instr[7:4]);
            else seg <= 8'hFF;
	 3: if (mode) seg <= bin2hex(Instr[3:0]) & `Dot;
            else seg <= 8'hFF;
	 4: if (mode) seg <= bin2hex({2'b00,PC[5:4]});
	         else seg <= bin2hex({2'b00,Addr[5:4]}); 
	 5: if (mode) seg <= bin2hex(PC[3:0]) & `Dot;
	         else seg <= bin2hex(Addr[3:0]) & `Dot;
	 6: if (mode) seg <= bin2hex(Accu[7:4]); // im CPU Modus Akku
            else seg <= bin2hex(Data[7:4]);      // im Edit Modus Data
	 7: if (mode) seg <= bin2hex(Accu[3:0]);
            else seg <= bin2hex(Data[3:0]);  
	endcase 
	
	Pos <= Pos + 1;
 end 
end 
 
// ----------
// Umwandlung binäre 4 Bit Zahl in Hex-Ziffer 
// ausgelagert in eine Funktion
function [7:0] bin2hex (input [3:0] x);
begin 
  case (x)
   4'h0: bin2hex =8'd192;
	4'h1: bin2hex = 8'd249;
	4'h2: bin2hex = 8'd164;
	4'h3: bin2hex = 8'd176;
	4'h4: bin2hex = 8'd153;
	4'h5: bin2hex = 8'd146;
	4'h6: bin2hex = 8'd130;
	4'h7: bin2hex = 8'd248;
	4'h8: bin2hex = 8'd128;
	4'h9: bin2hex = 8'd144;
	4'hA: bin2hex = 8'd136;
	4'hB: bin2hex = 8'd131;
	4'hC: bin2hex = 8'd198;
	4'hD: bin2hex = 8'd161;
	4'hE: bin2hex = 8'd134;
	4'hF: bin2hex = 8'd142;
  endcase
end  
endfunction 
 
endmodule 
