// Editor fuer MCPU System, zweiter Versuch

`define	state_rdA		3'b000
`define	state_rdB		3'b001
`define	state_keys		3'b010
`define	state_wrA		3'b011
`define	state_wrB		3'b100
`define	state_init		3'b101

module Editor (
 input 	 wire clk,
 input    wire mode,
 input	 wire	Adec,
 input 	 wire Ainc,
 input	 wire	DHinc,
 input	 wire	DLinc,
 input	 wire	init,
 inout	 wire	[7:0] ramData,
 output	 reg	nOE, nWE,
 output   reg  [7:0] Data,
 output   reg  [5:0] Addr
);

reg [2:0]	state;
reg [2:0]	state_return;
reg [6+1:0] counter;		// 1 Bit mehr als Adressbereich!

 always @ (posedge clk) begin
 if (mode == `EditMode) begin	// nur im Edit Mode was machen 
  case (state) 
  `state_rdA: begin
    nOE <= 0;		// Leseanforderung setzen
    nWE <= 1;
    state <= `state_rdB;
   end 
	
  `state_rdB: begin
    Data <= ramData;	// Lesevorgang
    nOE <= 1;
    nWE <= 1;
    state <= `state_keys;
   end

  `state_keys: begin 	
    if (Ainc) begin Addr <= Addr + 1; state <= `state_rdA; end  
    else if (Adec) begin Addr <= Addr - 1; state <= `state_rdA; end
    if (DHinc) begin 
	   Data[7:4] <= Data[7:4] + 1;
		state <= `state_wrA; state_return = `state_keys;
	 end
    else if (DLinc) begin
	   Data[3:0] <= Data[3:0] + 1; 
		state <= `state_wrA; state_return = `state_keys;
	 end
    if (init) state <= `state_init;
	end 
	
    `state_wrA: begin
     nOE <= 1;
     nWE <= 0;		// Schreibvorgang anmelden
     state <= `state_wrB;
    end
 
    `state_wrB: begin 
     nOE <= 1;
     nWE <= 1;		// Daten werden in dem Moment geschrieben, wenn nWE auf 1 geht
     state <= state_return;
    end 
	
    `state_init: begin	// 64 Byte SRAM initialisieren
     if (counter < 'h40) begin  
      Addr <= counter;
      Data <= 8'h00;
      state <= `state_wrA; state_return <= `state_init;
      counter <= counter + 1;
     end 
     else begin
      Addr <= 0;
      counter <= 0;
      state <= `state_rdA;	// go back to user interface
     end
    end 
	 
   endcase
  end // if mode == EditMode
 end // always 
 
endmodule
