/* Implementierung der MCPU:
   ein minimalistischer Prozessor zu Lehrzwecken */

// die Phasen bzw "Zustaende" des Prozessorzyklus:
`define  reqInstr  3'b000
`define  readInstr 3'b001
`define  reqArg	 3'b010
`define  readArg   3'b011
`define  waitExec  3'b100
`define  execInstr 3'b101

// die Maschinenbefehle der MCPU
`define  NOR	    2'b00
`define  ADD       2'b01
`define  STA       2'b10
`define  JCC       2'b11

module MCPU (
 input  wire clk,
 input  wire mode,
 input  wire reset,	// high aktives Reset
 input  wire sstep,	// single step key
 inout  wire [7:0] ramData,
 output reg  [5:0] Addr,
 output reg  [7:0] Data,
 output reg  Carry,	// das "Carry" Flag
 output reg  [7:0] Instr, // eingelesene Instruktion 
 output reg  [7:0] Accu,		// Akkumulator
 output reg  [5:0] PC,			// Befehlszeiger "Program Counter"
 output reg  nWE,	// SRAM Write Enable 
 output reg  nOE	// SRAM Output Enable 
);
 reg [2:0] NextState;// Prozessorzustand
 reg [7:0] Arg;			// eingelesenes Argument

 always @ (posedge clk) begin 
  if (mode == `CPUMode) begin	// nur im CPU Modus ueberhaupt was machen
  if (reset) begin	// Prozessor ruecksetzen
   Accu <= 0;
   PC <= 0;
   NextState <= `reqInstr;
  end
  else begin 			// normale Ausfuehrung
   case (NextState)
	 `reqInstr: begin	// 0) Instruktion anfordern
	   Addr <= PC;
		{nWE,nOE} <= 2'b10;
	   NextState <= `readInstr; end 

	 `readInstr: begin    // 1) Instruktion vom SRAM lesen
	   Instr <= ramData;	
		NextState <= `reqArg; end

    `reqArg: begin  // 2) Argument anfordern
	   Addr <= Instr[5:0];
	   {nWE,nOE} <= 2'b10;
	   NextState <= `readArg; end 
    `readArg: begin // 3) Argument vom SRAM lesen
        Arg <= ramData;
        NextState <= `waitExec;
       end  
    `waitExec: if (sstep) begin 
			 NextState <= `execInstr;
			 PC <= PC + 1;
         end
		 
	 `execInstr: begin 
	   case (Instr[7:6])
		 `NOR: Accu <= ~(Accu | Arg); 
		 `ADD: {Carry,Accu} <= Accu + Arg;
		 `STA: begin 
		   {nWE,nOE} <= 2'b01; ; Addr <= Instr[5:0]; Data <= Accu; end 
		 `JCC: begin
		   if (Carry == 0) PC <= Instr[5:0];
			else Carry <= 0; end 
		endcase
	   NextState <= `reqInstr; 
     end // execInstr
	 endcase
	end	// Ende der reset | normal Verzweigung 
   end // if mode = CPUMode
 end // always  
 
 endmodule // MCPU
 